package com.seeyon.ctp.organization.controller;

import com.seeyon.apps.collaboration.manager.ColManager;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.SystemEnvironment;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.controller.BaseController;
import com.seeyon.ctp.common.ctpenumnew.EnumNameEnum;
import com.seeyon.ctp.common.ctpenumnew.manager.EnumManager;
import com.seeyon.ctp.common.dao.paginate.Pagination;
import com.seeyon.ctp.common.datai18n.manager.DataI18nManager;
import com.seeyon.ctp.common.excel.DataRecord;
import com.seeyon.ctp.common.excel.DataRow;
import com.seeyon.ctp.common.excel.FileToExcelManager;
import com.seeyon.ctp.common.i18n.LocaleContext;
import com.seeyon.ctp.common.i18n.ResourceUtil;
import com.seeyon.ctp.common.i18n.expand.ResourceConsts;
import com.seeyon.ctp.common.i18n.util.I18nUtil;
import com.seeyon.ctp.common.po.ctpenumnew.CtpEnumItem;
import com.seeyon.ctp.organization.OrgConstants;
import com.seeyon.ctp.organization.OrgConstants.Role_NAME;
import com.seeyon.ctp.organization.bo.V3xOrgAccount;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.bo.V3xOrgPost;
import com.seeyon.ctp.organization.dao.OrgHelper;
import com.seeyon.ctp.organization.inexportutil.DataUtil;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.organization.manager.OrgManagerDirect;
import com.seeyon.ctp.organization.po.OrgMember;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.annotation.CheckRoleAccess;
import com.seeyon.ctp.util.json.JSONUtil;

import java.util.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.springframework.web.servlet.ModelAndView;

@CheckRoleAccess(
        roleTypes = {Role_NAME.GroupAdmin, Role_NAME.AccountAdministrator, Role_NAME.HrAdmin, Role_NAME.SystemAdmin}
)
public class PostController extends BaseController {
    protected OrgManager orgManager;
    protected OrgManagerDirect orgManagerDirect;
    protected FileToExcelManager fileToExcelManager;
    private DataI18nManager dataI18nManager;
    protected EnumManager enumManagerNew;

    private ColManager colManager;

    public ColManager getColManager() {
        if (this.colManager == null) {
            this.colManager = (ColManager)AppContext.getBean("colManager");
        }

        return this.colManager;
    }

    public void setColManager(ColManager colmanager) {
        this.colManager = colmanager;
    }

    public PostController() {
    }

    public OrgManager getOrgManager() {
        return this.orgManager;
    }

    public void setOrgManager(OrgManager orgManager) {
        this.orgManager = orgManager;
    }

    public OrgManagerDirect getOrgManagerDirect() {
        return this.orgManagerDirect;
    }

    public void setOrgManagerDirect(OrgManagerDirect orgManagerDirect) {
        this.orgManagerDirect = orgManagerDirect;
    }

    public FileToExcelManager getFileToExcelManager() {
        return this.fileToExcelManager;
    }

    public void setFileToExcelManager(FileToExcelManager fileToExcelManager) {
        this.fileToExcelManager = fileToExcelManager;
    }

    public void setDataI18nManager(DataI18nManager dataI18nManager) {
        this.dataI18nManager = dataI18nManager;
    }

    protected EnumManager getEnumManager() {
        if (this.enumManagerNew == null) {
            this.enumManagerNew = (EnumManager)AppContext.getBean("enumManagerNew");
        }

        return this.enumManagerNew;
    }

    public ModelAndView showPostframe(HttpServletRequest request, HttpServletResponse response) throws Exception {
        ModelAndView mav = new ModelAndView("apps/organization/post/listPost");
        Long accountId = AppContext.currentAccountId();
        String accountIdStr = request.getParameter("accountId");
        if (Strings.isNotBlank(accountIdStr)) {
            accountId = Long.valueOf(accountIdStr);
        }
        User user = AppContext.getCurrentUser();
        if(user.getName().equals("系统管理员")){
            List<OrgMember> acc = colManager.selectacc("单位管理员");
            OrgMember use= acc.get(0);
            accountId = use.getOrgAccountId();
        }
        mav.addObject("accountId", accountId);
        mav.addObject("isGroup", OrgConstants.GROUPID.equals(accountId));
        return mav;
    }

    public ModelAndView exportPost1(HttpServletRequest request, HttpServletResponse response) throws Exception {
        User u = AppContext.getCurrentUser();
        if (u == null) {
            return null;
        } else if (DataUtil.doingImpExp(u.getId())) {
            return null;
        } else {
            Long accountId = AppContext.currentAccountId();
            String accountIdStr = request.getParameter("accountId");
            if (Strings.isNotBlank(accountIdStr)) {
                accountId = Long.valueOf(accountIdStr);
            }

            boolean isGroup = OrgConstants.GROUPID.equals(accountId);
            String listname = ResourceUtil.getString("org.post_form.list");
            listname = listname + u.getLoginName();
            if (isGroup) {
                listname = ResourceUtil.getString("org.group_post_form.list");
            }

            DataUtil.putImpExpAction(u.getId(), "export");
            DataRecord dataRecord = null;

            try {
                dataRecord = this.exportPost(request, response, this.fileToExcelManager, this.orgManagerDirect, accountId, isGroup);
            } catch (Exception var10) {
                DataUtil.removeImpExpAction(u.getId());
                throw var10;
            }

            DataUtil.removeImpExpAction(u.getId());
            OrgHelper.exportToExcel(request, response, this.fileToExcelManager, listname, dataRecord);
            return null;
        }
    }

    private DataRecord exportPost(HttpServletRequest request, HttpServletResponse response, FileToExcelManager fileToExcelManager, OrgManagerDirect orgManagerDirect, Long accountId, boolean isGroup) throws Exception {
        DataRecord dataRecord = new DataRecord();
        Pagination.setNeedCount(false);
        String condition = request.getParameter("condition");
        Map paraMap = new HashMap();
        if (!"".equals(condition) && !"undefined".equals(condition)) {
            paraMap = (Map)JSONUtil.parseJSONString(condition);
        }

        List<V3xOrgPost> postlist = orgManagerDirect.getAllPosts(accountId, true);
        V3xOrgPost post = null;
        V3xOrgAccount account = null;
        String state_Enabled = ResourceUtil.getString("org.account_form.enable.use");
        String state_Disabled = ResourceUtil.getString("org.account_form.enable.unuse");
        String post_name = ResourceUtil.getString("org.post_form.name");
        StringBuilder post_i18n_name = new StringBuilder();
        List<Locale> allLocales = LocaleContext.getAllLocales();
        String[] locales = new String[allLocales.size()];

        String post_code;
        String post_state;
        for(int i = 0; i < allLocales.size(); ++i) {
            locales[i] = I18nUtil.getLocalAsString((Locale)allLocales.get(i));
            if (i > 0) {
                post_i18n_name.append(",");
            }

            post_code = I18nUtil.getLocalAsString((Locale)allLocales.get(i));
            post_state = ResourceConsts.getLanguageName(post_code);
            post_i18n_name.append(ResourceUtil.getString("org.post_form.name") + "(" + post_state + "(" + post_code + ")" + ")");
        }

        if (SystemEnvironment.functionSwitch("dataI18n")) {
            post_name = post_i18n_name.toString();
        }

        String post_type = ResourceUtil.getString("org.post_form.type");
        post_code = ResourceUtil.getString("org.post_form.type.code");
        post_state = ResourceUtil.getString("org.state.lable");
        String post_account = ResourceUtil.getString("org.account.lable");
        String post_list = ResourceUtil.getString("org.post_form.list");
        if (null != postlist && postlist.size() > 0) {
            DataRow[] datarow = new DataRow[postlist.size()];
            String con = "";
            String type = "";
            String value = "";
            if (((Map)paraMap).containsKey("type")) {
                type = ((Map)paraMap).get("type").toString();
            }

            if (((Map)paraMap).containsKey("condition")) {
                con = ((Map)paraMap).get("condition").toString();
            }

            if (((Map)paraMap).containsKey("value")) {
                value = ((Map)paraMap).get("value").toString();
            }

            int datarowIndex = 0;

            for(int i = 0; i < postlist.size(); ++i) {
                post = (V3xOrgPost)postlist.get(i);
                String postName = post.getName();
                String postCode = post.getCode();
                if (Strings.isNotBlank(type) && Strings.isNotBlank(con) && Strings.isNotBlank(value)) {
                    if ("input".equals(type)) {
                        if ("name".equals(con) && postName.indexOf(value) < 0 || "code".equals(con) && postCode.indexOf(value) < 0) {
                            continue;
                        }
                    } else if ("select".equals(type) && !value.equals(post.getTypeId().toString())) {
                        continue;
                    }
                }

                String typeName = "";
                DataRow row = new DataRow();
                CtpEnumItem item = this.getEnumManager().getEnumItem(EnumNameEnum.organization_post_types, post.getTypeId().toString());
                if (item != null) {
                    typeName = ResourceUtil.getString(item.getLabel());
                } else {
                    this.logger.info("post: " + post.getName() + " ,did not find post type");
                }

                if (SystemEnvironment.functionSwitch("dataI18n")) {
                    Map<String, String> map = null;
                    if (post.getI18nNameId() != null) {
                        map = this.dataI18nManager.getAllDataI18nValueMap(post.getI18nNameId());
                    }

                    String[] var36 = locales;
                    int var37 = locales.length;

                    for(int var38 = 0; var38 < var37; ++var38) {
                        String locale = var36[var38];
                        if (map != null && !map.isEmpty()) {
                            row.addDataCell((String)map.get(locale), 1);
                        } else {
                            row.addDataCell(post.getName(), 1);
                        }
                    }
                } else {
                    row.addDataCell(post.getName(), 1);
                }

                row.addDataCell(post.getCode(), 1);
                row.addDataCell(typeName, 1);
                if (!isGroup) {
                    account = this.orgManager.getAccountById(post.getOrgAccountId());
                    row.addDataCell(account.getName(), 1);
                }

                if (post.getEnabled()) {
                    row.addDataCell(state_Enabled, 1);
                } else {
                    row.addDataCell(state_Disabled, 1);
                }

                datarow[datarowIndex] = row;
                ++datarowIndex;
            }

            try {
                dataRecord.addDataRow(datarow);
            } catch (Exception var40) {
                ;
            }
        }

        String columnNames = "";
        if (isGroup) {
            columnNames = post_name + "," + post_code + "," + post_type + "," + post_state;
        } else {
            columnNames = post_name + "," + post_code + "," + post_type + "," + post_account + "," + post_state;
        }

        String[] columnName = columnNames.split(",");
        dataRecord.setColumnName(columnName);
        dataRecord.setTitle(post_list);
        short subHeight = 2000;
        dataRecord.setSubHeight(subHeight);
        if (SystemEnvironment.functionSwitch("dataI18n")) {
            dataRecord.setSubTitle(ResourceUtil.getString("org.post_form.list.desc.international"));
        } else {
            dataRecord.setSubTitle(ResourceUtil.getString("org.post_form.list.desc"));
        }

        dataRecord.setSheetName(post_list);
        return dataRecord;
    }
}

